

#ifndef GERTBOT_H
#define GERTBOT_H

#ifdef _WIN32

extern "C" {
#include "gertbot_winserial.h"
}

#define SLEEP_MS(x) Sleep(x)

#else

extern "C" {
#include "gertbot_pi_uart.h"
}

#include <unistd.h>
#define SLEEP_MS(x) usleep(x*1000)

#endif

#include <qglobal.h>

/**
#if QT_VERSION >= 0x050000
#define INCPATH QWidgets
#else
#define INCPATH QtGui 
#endif
**/

#include "assert.h"

#include "ui_gertbot.h"
#include "ui_board.h"
#include "ui_motor.h"
#include "gertbot_log.h"
#include "gertbot_dccsetup.h"

#include <qstring.h>
#include <qcheckbox.h>
#include <qcheckbox.h>
#include <QMainWindow>
#include <QButtonGroup>
#include <QCheckBox>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QGroupBox>
#include <QHeaderView>
#include <QLabel>
#include <QLineEdit>
#include <QMainWindow>
#include <QMenuBar>
#include <QPushButton>
#include <QToolButton>
#include <QStatusBar>
#include <QTabWidget>
#include <QTextEdit>
#include <QToolBar>
#include <QWidget>
#include <QTextCursor>
#include <QTimer>
#include <QLine>
#include <qmessagebox.h>
#include <stdint.h>

#include "gertbot_defines.h"

#define POLL_INTERVAL 1 // seconds between polls 

// Spec says 1/6 VDD ref to 5/6 VDD ref. 
// VDD rfe = ~3.3V thus
#define DAC_MAX 2.7
#define DAC_MIN 0.7

#define MOT_STEP_OFF_FLG 0x01 // Power off stepper when stopped 

// Motor widget stack pages 
#define MOT_PAGE_OFF          0 // blank
#define MOT_PAGE_BRUSH        1 // Freq. duty-cycle, move buttons
#define MOT_PAGE_DCC          2 // DCC buttons
#define MOT_PAGE_STEP         3 // Freq. steps, move & double stop buttons
#define MOT_PAGE_RMPSHRT      4 // Ramp up/dwn/halt , short/hot
#define MOT_PAGE_ENDSTOP      5 // Endstops
#define MOT_PAGE_DCCCONF      6 // DCC setup
#define MOT_PAGE_QUAD_ENB     7 // Quad enable
#define MOT_PAGE_QUAD_LIM     8 // Quad limits
#define MOT_PAGE_SERVO        9 // Servo controls

// Motor ramping stack pages
#define MOT_RAMP_BRUSH 0
#define MOT_RAMP_STEP  1



// Index of Gui PD controls
#define GUI_PD_INPUT  0 // controls is set to input
#define GUI_PD_OUTPUT 1 // controls is set to output
#define GUI_PD_FUNCT  2 // controls is set to function

#define PD_ADC0  10
#define PD_ADC1  11
#define PD_ADC2  12
#define PD_ADC3  13
#define PD_DAC1  15
#define PD_DAC0  14

#define DCC_TIMEOUT 35 // DCC slider time-out (empirical windows...)

#define MAXINT24 8388608
#define MININT24 -8388608

typedef uint8_t  byte;

extern uint32_t available_commports;

typedef struct {
  bool on;
  bool max_stop,min_stop;
  bool slow_on;
  int  position;
  uint32_t duty_cycle;
  int  max_pos,min_pos;
  int  goto_pos;
  int  slow_dist;
} quad_type;


typedef struct {
  byte mode;
  byte endmode;
  byte move;
  byte ramp_up;
  byte ramp_down;
  byte ramp_halt;
  byte short_hot_stop;
  uint32_t flags;
  uint32_t frequency;
  uint32_t duty_cycle;
  int32_t  steps;
  int32_t  remain;
  int32_t  missed;
  quad_type quad;
} gb_motor_type;


typedef struct {
  bool present;
  int  version;
  bool sync_mode;
  uint32_t adc_enable;
  uint32_t dac_enable;
  // ports are (LS->MS) EXT, spares, ADC, DAC, OD 
  uint32_t input_ports,output_ports;
  uint32_t inputs_status,outputs_status;
  uint32_t od;
  double adc[4]; // last read value 
  double dac[2]; // Last set value 
  int    attentionmode;
} gb_board_type;

extern gb_motor_type gb_motor[4][4];
extern gb_board_type board[4];

typedef struct {
//  QGroupBox *group;   // Group box
  QFrame    *motor_frame;
  Ui::motor *uipnt;
  QDoubleSpinBox *sb_dutycycle;
  int         mode;
  int         my_b,my_m; // which one am I ? 
  int         more_index;
} motor_gui_type;

typedef struct {
  QGroupBox *misc_group;
  QFrame    *box[20],
            *j3_frame;
  QCheckBox *j3_enb[20],
            *cb_od[2];
//  QLabel    *j3_lbl[20];
  QCheckBox   *cb_synchronous;
  QPushButton *pb_motor_to_board,
              *pb_board_to_motor,
              *pb_read;
  QComboBox *attention;
} board_gui_type;

typedef struct {
  QComboBox *pd[16];
  QCheckBox *cb[16];
  QLabel    *lbl[16];
  QDoubleSpinBox *sb_dac[2];
  QLabel    *lbl_adc[4];

} j3_gui_type;

extern gb_motor_type gb_motor[4][4];
extern int index_to_mode[8];

void uart_tx(byte);
void uart_tx(byte,byte);
void uart_tx(byte,byte,byte);
void uart_tx(byte,byte,byte,byte);
void uart_tx(byte,byte,byte,byte,byte);
void uart_tx(byte *buf,int length,int pad);
int find_board(int b,int *ver_ms,int *ver_ls);
int get_reply(unsigned char *data,   // received re-ply
              int byte_count,        // number of bytes we expect
              unsigned char id,      // Board id
              unsigned char command);// command sent

extern bool uart_opened;
extern int log_show_reply;
extern int build_int;

class gertbot : public QMainWindow
{
    Q_OBJECT

public:
    gertbot(QWidget *parent = 0, Qt::WFlags flags = 0);
    ~gertbot();
    void add_log(QString str);
    gertbot_log *log_window;

private:
    Ui::gertbotClass ui;
    Ui::board  boardui[4];
    j3_gui_type j3_gui[4];    
    dccsetup_type dcc_gui[4][4];
//    void set_motormode_gui(byte ,byte, byte);
    bool make_motor_group(int board,int mot);
    void set_motormode_gui(int board,int mot);
    void run_slot(int);
    void set_gui_from_motor(int b,int m);
    void motor_buttons_on(int board,int mot,bool on);
    void make_board_gui(int b);
    void board_mot_slt(int b);
    void adc_dac_enable(int board);
    void configure_system();
    void send_endmode(int b,int m);
    void send_shorthot(int b,int m);
    void send_freq(int b,int m);
    void send_dutycycle(int b,int m);
    void send_steps(int b,int m);
    void send_run(int b,int m,int but);
    void board_set_mode(int b);
    void j3pd_endpoint(int b,int p,bool m);
    void j3pd_spare(int b,int p);
    void j3pd_adc(int b,int p);
    void j3pd_dac(int b,int p);
    void j3_set_input(int b, int p);
    void j3_set_output(int b,int p);
    void set_port_out_high(int b); 
    void set_io(int b);
    float adc_read(int brd,int adc);
    void make_motor_switch(int board);
    int  input_read(int b);
    void set_stop_buttons(int);
    bool get_motor_config(int brd,int mot,gb_motor_type *status);
    bool get_motor_status(int brd,int mot,gb_motor_type *status);
    bool get_motor_missed(int brd,int mot,gb_motor_type *status);
    bool get_board_status(int brd,gb_board_type *status);
    bool get_io_status(int brd,gb_board_type *status);
    void set_gui_from_board(int b) ;
    void dcc_init();
    void set_dcc_gui(int,int,dccsetup_type *setup);
    void j3_set_input_cb(int b, int p, int high);
    void end_stop_gui(int b,int m);
    QString get_build_date_time(int brd);
// Quad 
    void quad_on_message(int brd,int mot,int on);
    void set_limit_gui(Ui::motor *mtrptr);
    void need_sw_update(QString);
// Step ramp
    void send_stepramp(int b,int m);


// GUI objects
    motor_gui_type motor_gui[4][4];
    board_gui_type board_gui[4];
    QFrame  *frm_board[4];
    QWidget *tab[4];
    bool suppress_updates;
    bool gui_disp_motor[4];
    bool polling;
    int  poll_time;
    QTimer timer;
    QTimer dcc_timer;
    QIcon icon_left,icon_right,icon_pause,icon_stop;
    QIcon icon_left_grn,icon_right_grn,icon_pause_grn,icon_stop_grn;
    bool  open_log_on_first_connect;
    int   text_display[4];
    bool  force_command; // make UART commands go out 
    int   slider_data[2];
    bool  sim_mode;



public slots:
  void pd_mode_slot();
  void endmode_slot();
  void freq_slot();
  void dutycycle_slot();
  void steps_slot();
  void runA_slot();
  void runS_slot();
  void runB_slot();
  void runI_slot();
  void connect_slot();
  void stop_all_slot();
  void halt_slot();
  void read_status_slot();
  void read_errors_slot();
  void sync_mode_slot(int);
  void do_sync_slot();
  void board_mot_slot();
  void dac_enable_slot();
  void dac_value_slot();
  void adc_enable_slot();
  void log_slot(bool);
  void log_close_slot();
  void update_slot();
  void poll_slot(bool);
  void timer_slot();
  void short_slot(int);
  void poll_time_slot(double);
  void j3pd_slot();
  void j3cb_slot(int);
  void opendrain_slot();
  void board_read_slot();
  void attention_slot(int);
  void board_pin_slot();
  void user_send_slot();
  void more_slot();
  void dccsetup_slot();
  void dcc_but_slot(); 
  void dcc_timer_slot();
  void dcc_slide_slot();
  void ramp_slot();
  void dcc_config_slot();
  void baud_slot(int);
  void sim_slot();
// Quad
  void quad_on_slot(int);
  void quad_setpos_slot();
  void quad_read_slot();
  void quad_limit_slot();
  void quad_goto_slot();
// Servo
  void servo_pos_slot();
  void servo_config_slot();
// Step ramping
  void step_ramp_slot();


};

extern gertbot *gertbot_pnt;

// debug
extern int serial_show_return;


#endif // GERTBOT_H
