//
// Quadrature encoder code 
//

#include "gertbot.h"

void gertbot::quad_on_slot(int on)
{ QObject *qto = (QObject *)sender();
  int brd,mot;
  Ui::motor *mtrptr; // 
  if (suppress_updates)
    return; 
  brd = qto->objectName()[0].digitValue();
  mot = qto->objectName()[1].digitValue();
  mtrptr = motor_gui[brd][mot].uipnt;
  if (on && board[brd].version<205)
    // Board needs SW upgrade for quadrature encoder
    need_sw_update("Quadratur encoder");
  gb_motor[brd][mot].quad.on = on;
  if (on)
  { mtrptr->quad_goto_stack->setCurrentIndex(1);
    mtrptr->le_quad_setpos->setEnabled(true);
    mtrptr->sl_quad_setpos->setEnabled(true);
    mtrptr->pb_quad_set->setEnabled(true);
    mtrptr->pb_quad_read->setEnabled(true);
    mtrptr->lbl_position->setEnabled(true);
    mtrptr->lbl_errors->setEnabled(true);
    mtrptr->cb_quad_invert->setEnabled(true);
  }
  else
  { mtrptr->quad_goto_stack->setCurrentIndex(0);
    mtrptr->le_quad_setpos->setEnabled(false);
    mtrptr->sl_quad_setpos->setEnabled(false);
    mtrptr->pb_quad_set->setEnabled(false);
    mtrptr->pb_quad_read->setEnabled(false);
    mtrptr->lbl_position->setEnabled(false);
    mtrptr->lbl_errors->setEnabled(false);
    mtrptr->cb_quad_invert->setEnabled(false);
  }
  quad_on_message(brd,mot,on);
} // quad_on_slot


void gertbot::quad_on_message(int brd,int mot,int on)
{ byte mess[14],id;
  bool ok;
  int position;
  Ui::motor *mtrptr; // 
  mtrptr = motor_gui[brd][mot].uipnt;
  id = (brd<<2) | mot;
  mess[0]=CMD_QUAD;
  mess[1]=id;
  mess[2]  =mtrptr->cb_quad_invert->isChecked() ? 0x01 : 0x00;
  if (on)
    mess[2] |= QUAD_ON;
  
  // Pick up other flags which might have been set previously
  // (seq is: quad-on, change max/min/slow, quad off, quad-on)
  mess[2] |= mtrptr->cb_quad_max_on ->isChecked() ? QUAD_TOP : 0x00;
  mess[2] |= mtrptr->cb_quad_min_on ->isChecked() ? QUAD_BOT : 0x00;
  mess[2] |= mtrptr->cb_quad_slow_on->isChecked() ? QUAD_GOSLOW : 0x00;
   
  position = mtrptr->le_quad_setpos->text().toInt(&ok);
  if (!ok || position<MININT24 || position>MAXINT24)
  { // Illegal value
    mtrptr->le_quad_setpos->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
    return;
  }
  mtrptr->le_quad_setpos->setStyleSheet("QLineEdit { background-color : white; color : black; }");
  mess[3] = (byte)((position>>16)& 0xFF);
  mess[4] = (byte)((position>>8)& 0xFF);
  mess[5] = (byte)( position    & 0xFF);
  uart_tx(mess,6,0);
} // quad_on_message

void gertbot::quad_setpos_slot()
{ QObject *qto = (QObject *)sender();
  int brd,mot;
  if (suppress_updates)
    return; 
  brd = qto->objectName()[0].digitValue();
  mot = qto->objectName()[1].digitValue();
  quad_on_message(brd,mot,1);
} // quad_setpos_slot

void gertbot::quad_read_slot()
{ byte id,rec[16],cmd[2];
  QString str;
  int     val;
  QObject *qto = (QObject *)sender();
  int brd,mot;
  if (suppress_updates)
    return; 
  brd = qto->objectName()[0].digitValue();
  mot = qto->objectName()[1].digitValue();
 
  id = (brd<<2) | mot;
  cmd[0] = CMD_QUAD_READ;
  cmd[1] = id;
  uart_tx(cmd,2,8);
  // Get reply...
  if (!get_reply(rec,7,id,(byte)CMD_QUAD_READ))
  { str.sprintf("%d:No reply on quad. read %d",brd,mot);
    add_log(str);
    return;
  }
  val  = (int)rec[2]<<16;
  val |= (int)rec[3]<<8;
  val |= (int)rec[4];
  if (val & 0x00800000)
    val |= 0xFF000000;
  str.sprintf("Position: %d",val);
  motor_gui[brd][mot].uipnt->lbl_position->setText(str);

  val  = (int)rec[5]<<8;
  val |= (int)rec[6];
  str.sprintf("Error(s): %d",val);
  motor_gui[brd][mot].uipnt->lbl_errors->setText(str);
  
} // quad_read_slot


//
// Enable/disable controls
//
void gertbot::set_limit_gui(Ui::motor *mtrptr)
{ if (mtrptr->cb_quad_max_on->isChecked())
  { mtrptr->le_maxquad->setEnabled(true);
    mtrptr->sl_quad_max->setEnabled(true);
  }
  else
  { mtrptr->le_maxquad->setEnabled(false);
    mtrptr->sl_quad_max->setEnabled(false);
  }
  if (mtrptr->cb_quad_min_on->isChecked())
  { mtrptr->le_minquad->setEnabled(true);
    mtrptr->sl_quad_min->setEnabled(true);
  }
  else
  { mtrptr->le_minquad->setEnabled(false);
    mtrptr->sl_quad_min->setEnabled(false);
  }
  if (mtrptr->cb_quad_slow_on->isChecked())
  { mtrptr->le_quad_dist->setEnabled(true);
    mtrptr->sl_quad_slowdist->setEnabled(true);
    mtrptr->sb_quad_slowdc->setEnabled(true);
    mtrptr->sl_quad_slowdc->setEnabled(true);
  }
  else
  { mtrptr->le_quad_dist->setEnabled(false);
    mtrptr->sl_quad_slowdist->setEnabled(false);
    mtrptr->sb_quad_slowdc->setEnabled(false);
    mtrptr->sl_quad_slowdc->setEnabled(false);
  }
} // set_limit_gui


void gertbot::quad_limit_slot()
{ QObject *qto = (QObject *)sender();
  int brd,mot,int_dc;
  bool ok;
  int max,min,dist;
  byte mess[14];
  double dbl_dc;
  Ui::motor *mtrptr; // 
  if (suppress_updates)
    return; 
  brd = qto->objectName()[0].digitValue();
  mot = qto->objectName()[1].digitValue();
  mtrptr = motor_gui[brd][mot].uipnt;
  set_limit_gui(mtrptr);


  mess[0]=CMD_QUAD_LIMIT;
  mess[1]=(brd<<2) | mot;
  mess[2]  = mtrptr->cb_quad_max_on ->isChecked() ? QUAD_TOP : 0x00;
  mess[2] |= mtrptr->cb_quad_min_on ->isChecked() ? QUAD_BOT : 0x00;
  mess[2] |= mtrptr->cb_quad_slow_on->isChecked() ? QUAD_GOSLOW : 0x00;
  if (mtrptr->cb_quad_max_on->isChecked())
  {
    max = mtrptr->le_maxquad->text().toInt(&ok);
    if (!ok || max<MININT24 || max>MAXINT24)
    { // Illegal value
      mtrptr->le_maxquad->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
      return;
    }
    mtrptr->le_maxquad->setStyleSheet("QLineEdit { background-color : white; color : black; }");
  }
  else
    max = 0;
  mess[3] = (byte)((max>>16)& 0xFF);
  mess[4] = (byte)((max>>8)& 0xFF);
  mess[5] = (byte)( max    & 0xFF);

  if (mtrptr->cb_quad_min_on->isChecked())
  {
    min = mtrptr->le_minquad->text().toInt(&ok);
    if (!ok || min<MININT24 || min>MAXINT24)
    { // Illegal value
      mtrptr->le_minquad->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
      return;
    }
    mtrptr->le_minquad->setStyleSheet("QLineEdit { background-color : white; color : black; }");
  }
  else
    min = 0;
  mess[6] = (byte)((min>>16)& 0xFF);
  mess[7] = (byte)((min>>8)& 0xFF);
  mess[8] = (byte)( min    & 0xFF);

  if (mtrptr->cb_quad_slow_on->isChecked())
  {
    dist = mtrptr->le_quad_dist->text().toInt(&ok);
    if (!ok || dist<0 || dist>65535)
    { // Illegal value
      mtrptr->le_quad_dist->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
      return;
    }
    mtrptr->le_quad_dist->setStyleSheet("QLineEdit { background-color : white; color : black; }");
  }
  else
    dist = 0;
  mess[9] = (byte)((dist>>8)& 0xFF);
  mess[10] = (byte)( dist    & 0xFF);

  dbl_dc = mtrptr->sb_quad_slowdc->value();
  int_dc = (int)(dbl_dc*10);
  if (int_dc<0 || int_dc>1000)
  { // Illegal value
    mtrptr->sb_quad_slowdc->setStyleSheet("QDoubleSpinBox { background-color : yellow; color : red; }");
    return;
  }
  mtrptr->sb_quad_slowdc->setStyleSheet("QDoubleSpinBox { background-color : white; color : black; }");
  mess[11] = (byte)((int_dc>>8)& 0xFF);
  mess[12] = (byte)( int_dc    & 0xFF);

  uart_tx(mess,13,0);
} // quad_limit_slot

void gertbot::quad_goto_slot()
{ QObject *qto = (QObject *)sender();
  int brd,mot,int_dc;
  bool ok;
  int dest;
  byte mess[14];
  double dbl_dc;
  Ui::motor *mtrptr; // 
  if (suppress_updates)
    return; 
  brd = qto->objectName()[0].digitValue();
  mot = qto->objectName()[1].digitValue();
  mtrptr = motor_gui[brd][mot].uipnt;
  mess[0]=CMD_QUAD_GOTO;
  mess[1]=(brd<<2) | mot;;

  dest = mtrptr->le_quad_gotopos->text().toInt(&ok);
  if (!ok || dest<MININT24 || dest>MAXINT24)
  { // Illegal value
    mtrptr->le_quad_gotopos->setStyleSheet("QLineEdit { background-color : yellow; color : red; }");
    return;
  }
  mtrptr->le_quad_gotopos->setStyleSheet("QLineEdit { background-color : white; color : black; }");
  mess[2] = (byte)((dest>>16)& 0xFF);
  mess[3] = (byte)((dest>>8)& 0xFF);
  mess[4] = (byte)( dest    & 0xFF);

  dbl_dc = mtrptr->sb_quad_gotodc->value();
  int_dc = (int)(dbl_dc*10);
  if (int_dc<0 || int_dc>1000)
  { // Illegal value
    mtrptr->sb_quad_gotodc->setStyleSheet("QDoubleSpinBox { background-color : yellow; color : red; }");
    return;
  }
  mtrptr->sb_quad_gotodc->setStyleSheet("QLineEdit { background-color : white; color : black; }");
  mess[5] = (byte)((int_dc>>8)& 0xFF);
  mess[6] = (byte)( int_dc    & 0xFF);

  uart_tx(mess,7,0);
} // quad_goto_slot


